# Multiple Imputation
# November 3, 2015
# StatLab@UVa Library
# Michele Claibourn


# Set the working directory -- wherever you put the script and data files
setwd("Box Sync/mpc/DataServices/Workshops/MissingDataWorkshop/")

# Load the packages we'll use
#install.packages("mice")
#install.packages("VIM")
#install.packages("lattice")
library(mice)
library(VIM)
library(lattice)


######################################
## 1. Missing Data: Problems, Concepts
######################################
## Types/Costs of missingness
# MCAR, loss of power
# Generate random data: x1-x3 from normals; x4 from Bernoulli
# y is sum of x's, plus normal error term
set.seed(1017)
ex <- data.frame(x1=rnorm(1000), 
                 x2=rnorm(1000),
                 x3=rnorm(1000))
ex$y <- ex$x1 + ex$x2 + ex$x3 + 8*rnorm(1000)
summary(ex)
par(mfrow = c(2, 2)) # set to 2 x 2 frame 
hist(ex$x1, xlim=c(-4,4))
hist(ex$x2, xlim=c(-4,4))
hist(ex$x3, xlim=c(-4,4))
hist(ex$y)
par(mfrow = c(1, 1)) # reset to 1 x 1

# "True" model
reg.ex <- lm(y ~ x1 + x2 + x3, data=ex)
summary(reg.ex)

# Drop X values at random, 10% of each variable
ex.mcar <- ex
set.seed(1017)
ex.mcar$x1[sample(nrow(ex.mcar), 100)] <- NA
ex.mcar$x2[sample(nrow(ex.mcar), 100)] <- NA
ex.mcar$x3[sample(nrow(ex.mcar), 100)] <- NA
summary(ex.mcar)

# How many complete cases remaining?
ex.complete <- complete.cases(ex.mcar)
sum(ex.complete) # number of complete cases

# Graph densities of original data compared to data with obs mcar
par(mfrow = c(2, 2)) # set to 2 x 2 frame 
plot(density(ex$x1))
lines(density(ex.mcar$x1, na.rm=T), col="red", lty=2)
plot(density(ex$x2))
lines(density(ex.mcar$x2, na.rm=T), col="red", lty=2)
plot(density(ex$x3))
lines(density(ex.mcar$x3, na.rm=T), col="red", lty=2)
plot(density(ex$y))
lines(density(ex.mcar$y, na.rm=T), col="red", lty=2)
par(mfrow = c(1, 1)) # reset to 1 x 1

# Model using listwise deletion
reg.ex.mcar <- lm(y ~ x1 + x2 + x3, data=ex.mcar)
summary(reg.ex.mcar)
summary(reg.ex)


# MAR, biased estimates
# Drop values of x1, dependent on y (observed)
ex.mar <- ex
summary(ex.mar$y)
set.seed(121)
ex.mar$drop1 <- runif(1000)
summary(ex.mar$drop1)
ex.mar$x1 <- ifelse(ex.mar$drop1<ex.mar$y, NA, ex.mar$x1)
summary(ex.mar$x1)
summary(ex.mar$y)

# Graph densities of original data compared to data with obs mar
plot(density(ex$x1))
lines(density(ex.mar$x1, na.rm=T), col="red", lty=2)
plot(density(ex$y))
lines(density(ex.mar$y, na.rm=T), col="red", lty=2)

# Model using listwise deletion
reg.ex.mar <- lm(y ~ x1 + x2 + x3, data=ex.mar)
summary(reg.ex.mar)
summary(reg.ex)


# Simple imputation
# Drop Y values at random, about 10%
ex.mcar.mean <- ex.mcar
set.seed(823)
ex.mcar.mean$y[sample(nrow(ex.mcar.mean), 200)] <- NA

# Replace missing Y with mean Y
ex.mcar.mean$ynew <- ifelse(is.na(ex.mcar.mean$y), mean(ex.mcar.mean$y, na.rm=T), ex.mcar.mean$y)
ex.mcar.mean$x1new <- ifelse(is.na(ex.mcar.mean$x1), mean(ex.mcar.mean$x1, na.rm=T), ex.mcar.mean$x1)
ex.mcar.mean$x2new <- ifelse(is.na(ex.mcar.mean$x2), mean(ex.mcar.mean$x2, na.rm=T), ex.mcar.mean$x2)
ex.mcar.mean$x3new <- ifelse(is.na(ex.mcar.mean$x3), mean(ex.mcar.mean$x3, na.rm=T), ex.mcar.mean$x3)

plot(density(ex.mcar.mean$y, na.rm=T), ylim=c(0,.1),
     main="Mean Imputation Reduces Variance")
lines(density(ex.mcar.mean$ynew), col="red")

plot(y ~ x1, data=ex.mcar.mean, main="Mean Imputation Reduces Variance")
points(ex.mcar.mean$x1[is.na(ex.mcar.mean$y)], 
       ex.mcar.mean$ynew[is.na(ex.mcar.mean$y)], col="red")
points(ex.mcar.mean$x1new[is.na(ex.mcar.mean$x1)],
       ex.mcar.mean$y[is.na(ex.mcar.mean$x1)], col="blue")

reg.ex.mean <- lm(ynew ~ x1new + x2new + x3new, data=ex.mcar.mean)
summary(reg.ex.mean)
summary(reg.ex)


###################################
## 2. Preview: Improvement with MI
###################################
# Impute the missing values (we'll come back to this in detail later)
# First, note the imputation models themselves are not very predictive
x1.mod <- lm(x1 ~ x2 + x3 + y, data=ex.mcar) # For example
summary(x1.mod)

# Impute missing X from mcar data
imps.mcar <- mice(ex.mcar, m=5)
# Results after multiple imputation
reg.mi.mcar <- with(imps.mcar, lm(y ~ x1 + x2 + x3))
summary(pool(reg.mi.mcar))
# compared to "true" results
summary(reg.ex)

# Impute missing X from mar data
imps.mar <- mice(ex.mar, m=5)
# Results after multiple imputation
reg.mi.mar <- with(imps.mar, lm(y ~ x1 + x2 + x3))
summary(pool(reg.mi.mar))
# compared to "true" results
summary(reg.ex)

# Tidy up
rm(list=ls())


###########################
## 3. Exploring missingness
###########################
# Load some data
titanic <- read.table("titanic1.txt", sep=',', 
                      header=T, as.is="name",
                      na.strings=c("", NA))
head(titanic)
str(titanic)
summary(titanic)

# If there are other characters that represent missing, besides NA,
# then add the argument to read the indicator properly: 
#    na.strings="." (where period indicates missing)
# If missing has been captured with different characters, then add:
#    na.strings=c(".", "", "-")
# Or recode missing values to NA after the fact
# ex: dataframe$v1[dataframe$v1==99] <- NA 

# is.na returns TRUE if value is missing
miss1 <- is.na(titanic$age) 
str(miss1) # a logical vector
sum(miss1) # number missing
missclass <- table(titanic$pclass[miss1]) # table of pclass for obs where miss1=TRUE
missclass
totclass <- table(titanic$pclass)
propmissclass <- missclass/totclass
propmissclass

# complete.cases() returns a *logical vector* indicating which cases are complete
titanic.complete <- complete.cases(titanic)
sum(titanic.complete) # number of complete cases

# Arithmetic functions on vectors with missing values produces missing value
mean(titanic$age)
mean(titanic$age, na.rm=TRUE)

# Create a vector of variables to examine
incvars <- c("pclass", "survived", "age", "embarked", "home.dest", "sex")
titanic.complete <- complete.cases(titanic[, incvars])
sum(titanic.complete) # number of complete cases for included variables

titanic.sub <- subset(titanic, select=c("survived", "pclass", "sex", 
                                        "age", "embarked"))

# Examine missingness
md.pattern(titanic.sub)

# Examine visually with the VIM package
# Aggregation plot: Produces barplot of proportion missing; and frequency of missing combinations
aggr(titanic.sub, numbers=TRUE, prop = c(TRUE, FALSE))

# Spinogram: extension of histogram, plots distribution of a x1 against x2 
spineMiss(titanic.sub[, c("pclass", "age")])
spineMiss(titanic.sub[, c("pclass", "embarked")])
spineMiss(titanic.sub[, c("survived", "age")])

# Matrixplot: red is missing; darker values are higher
matrixplot(titanic.sub, sortby = "age")

# Also marginplot and scattmatrixMiss for combinations of continuous distributions


###########################
## 4. Implementing MI/MICE
###########################
# Create imputed datasets
titanic.imp <- mice(titanic.sub) # with defaults
# mice function returns object of class mids (Multiply Imputed Data Sets)
titanic.imp
str(titanic.imp)

# Missing values in titanic are filled in for the imputed data sets
complete(titanic.imp, 0)[10:20,]
complete(titanic.imp, 1)[10:20,]
complete(titanic.imp, 2)[10:20,]

summary(complete(titanic.imp, 0)$age)
summary(complete(titanic.imp, 1)$age)
summary(complete(titanic.imp, 5)$age)

# Some diagnostic plots (using lattice graphics)
plot(titanic.imp, layout=c(2,2))
stripplot(titanic.imp, pch=20)

# Density plot for imputed age
densityplot(titanic.imp)

# Let's change the imputation method for age, embarked; add more imputations and interations
titanic.imp2 <- mice(titanic.sub, m=10, maxit=25, 
             method=c(" ", " ", ", ", "norm", "lda")) 
titanic.imp2
stripplot(titanic.imp2, pch=20)
plot(titanic.imp2, layout=c(2,2))
densityplot(titanic.imp2)

###########################################
## 5. Analyze Imputed Data and Pool Results
###########################################
# Analyze and Pool
titanic.mod <- with(titanic.imp2, 
                    glm(survived ~ pclass*sex + age, family=binomial))
summary(pool(titanic.mod))
summary(pool(titanic.mod))[,c(1:3,5)]
# Tada!


#############################
## 6. For more complex models
#############################
# If the model includes interactions derived from variables that are imputed,
# Use "passive imputation"
head(titanic.imp$pad$data) # Note mice stores factors as series of dummies
titanic.imp # Note the predictor matrix indicates what variables are used to impute missing values for each X

# Attach interaction terms: interaction between age and sex
titanic.sub2 <- cbind(titanic.sub, age.sex.1=NA)
# Initialize mids object without running
titanic.imp3.init <- mice(titanic.sub2, max=0, print=F)
# Extract default methods from the above dry run
meth <- titanic.imp3.init$meth
meth
# Now change some of these
meth["age.sex.1"] <- "~I((age-31)*sex.1)"
meth["age"] <- "norm"
meth
# Extract default predict matrix from dry run
pred <- titanic.imp3.init$pred
pred
# Change to (1) insure that the interaction is not used 
# as predictor in the model for age
# but is used as predictor for other equations
pred["age", "age.sex.1"] <- 0
pred["embarked", "age.sex.1"] <- 1
pred

# Create imputations and check
titanic.imp3 <- mice(titanic.sub2, meth=meth, 
                    pred=pred, m=10, maxit=10)
plot(titanic.imp3)
stripplot(titanic.imp3)

# Analyze imputed data and pool
titanic.mod2 <- with(data=titanic.imp3, exp=glm(survived ~ pclass +I(age-31)*sex, 
                                               family=binomial))
summary(pool(titanic.mod2))

# For comparison
titanic.test <- glm(survived ~ pclass + I(age-31)*sex, data=titanic,
                    family=binomial)

round(summary(titanic.test)$coefficients, 3)
round(summary(pool(titanic.mod2))[, c(1:3,5)], 3)

# Tidy up
rm(list=ls())
